/********************************************************
*														*
* File:		confobject.cpp								*
* Author:	Sascha Kiefer								*
*														*
* Update:	2003-06-23									*
*			Umstellung auf SubSections					*
*														*
********************************************************/

#include "confobject.h"
#include "debug.h"

ConfObject::ConfObject()
{
}

ConfObject::~ConfObject()
{
	m_sections.clear();
}

void ConfObject::Insert(const string & section, const string & subsection, const string & key, const string & value)
{
	KeyMap km;
	SubSections sub;

	SectionsIter sec_iter = m_sections.find(section);
	if(sec_iter != m_sections.end()) sub = sec_iter->second;

	SubSectionsIter sub_iter = sub.find(subsection);
	if(sub_iter != sub.end()) km = sub_iter->second;

	km[key].push_back(value);
	sub[subsection] = km;
	m_sections[section] = sub;
}

bool ConfObject::Exist(const string & section, const string & key)
{
	return Exist(section, "", key);
}


bool ConfObject::Exist(const string & section, const string & subsection, const string & key)
{
	SectionsIter sec_iter = m_sections.find(section);
	if(sec_iter != m_sections.end())
	{
		if(subsection == "" && key == "") return true;
		SubSectionsIter sub_iter = sec_iter->second.find(subsection);
		if(sub_iter != sec_iter->second.end())
		{
			if(key == "") return true;
			KeyMapIter key_iter = sub_iter->second.find(key);
			if(key_iter != sub_iter->second.end()) return true;
		}
	}

	return false;
}

string ConfObject::Lookup(const string & section, const string & key)
{
	return Lookup(section, "", key);
}

string ConfObject::Lookup(const string & section, const string & subsection, const string & key)
{
	return Lookup(section, subsection, key, 0);
}

string ConfObject::Lookup(const string & section, const string & key, unsigned int idx)
{
	return Lookup(section, "", key, idx);
}

string ConfObject::Lookup(const string & section, const string & subsection, const string & key, unsigned int idx)
{
	if(section == "" || key == "") return "";

	SectionsIter sec_iter = m_sections.find(section);
	if(sec_iter == m_sections.end()) return "";

	SubSectionsIter sub_iter = sec_iter->second.find(subsection);
	if(sub_iter == sec_iter->second.end()) return "";

	KeyMapIter key_iter = sub_iter->second.find(key);
	if(key_iter == sub_iter->second.end()) return "";

	if(idx > key_iter->second.size()-1) return "";

	return key_iter->second[idx];
}

const KeyMap & ConfObject::GetKeymap(const string & section, const string & subsection)
{
	static const KeyMap emptykeymap;

	if(section == "") return emptykeymap;

	SectionsIter sec_iter = m_sections.find(section);
	if(sec_iter == m_sections.end()) return emptykeymap;

	SubSectionsIter sub_iter = sec_iter->second.find(subsection);
	if(sub_iter == sec_iter->second.end()) return emptykeymap;

	return sub_iter->second;
}

size_t ConfObject::GetValueCount(const string & section, const string & key)
{
	return GetValueCount(section, "", key);
}


size_t ConfObject::GetValueCount(const string & section, const string & subsection, const string & key)
{
	if(section == "" || key == "") return 0;

	SectionsIter sec_iter = m_sections.find(section);
	if(sec_iter == m_sections.end()) return 0;

	SubSectionsIter sub_iter = sec_iter->second.find(subsection);
	if(sub_iter == sec_iter->second.end()) return 0;

	KeyMapIter key_iter = sub_iter->second.find(key);
	if(key_iter == sub_iter->second.end()) return 0;

	return key_iter->second.size();
}


void ConfObject::Dump()
{
	SectionsIter sec_iter = m_sections.begin();

	while(sec_iter != m_sections.end())
	{
		std::cout << "[" << sec_iter->first << "]" << std::endl;

		SubSectionsIter sub_iter = sec_iter->second.begin();

		while(sub_iter != sec_iter->second.end())
		{
			if(sub_iter->first != "") std::cout << "<" << sub_iter->first << ">" << std::endl;

			KeyMapIter key_iter = sub_iter->second.begin();

			while(key_iter != sub_iter->second.end())
			{
				std::cout << key_iter->first << " = ";

				StringVecIter vec_iter = key_iter->second.begin();
				while(vec_iter != key_iter->second.end())
				{
					if(vec_iter != key_iter->second.begin()) std::cout << ",";
						std::cout << *vec_iter;
					++vec_iter;
				}

				std::cout << std::endl;
				++key_iter;
			}
			++sub_iter;
		}
		++sec_iter;
	}
}

ConfObject::ConfObject(ConfObject & co)
{
}

