#ifndef __CLASS_CONFOBJECT__
#define __CLASS_CONFOBJECT__

#include "plattform.h"
#include <iostream>
#include <string>
#include <vector>
#include <map>

#include "defines.h"

typedef std::string						string;

typedef std::vector<string>				StringVec;
typedef StringVec::iterator				StringVecIter;

typedef std::map<string, StringVec>		KeyMap;
typedef KeyMap::iterator               	KeyMapIter;
typedef std::pair<string, StringVec>	KeyMapPair;

typedef std::map<string, KeyMap>    	SubSections;
typedef SubSections::iterator           SubSectionsIter;
typedef std::pair<string, KeyMap>		SubSectionsPair;

typedef std::map<string, SubSections>  	Sections;
typedef Sections::iterator             	SectionsIter;
typedef std::pair<string, SubSections>	SectionsPair;


//! Die Klasse ConfObject reprsentiert die Konfigurationsdatei innerhalb des Programms.
/*! Sie stellt Methoden fr das Nachschlagen verschiedener Schlssel-Wert-Paare zur Verfgung. */
class ConfObject
{
	public:
    	ConfObject();
    	~ConfObject();

    	bool Exist(const string & section, const string & key);
    	bool Exist(const string & section, const string & subsection, const string & key);

    	bool ExistSubsections(const string & section);

    	string Lookup(const string & section, const string & key);
    	string Lookup(const string & section, const string & key, unsigned int idx);
    	string Lookup(const string & section, const string & subsection, const string & key);
    	string Lookup(const string & section, const string & subsection, const string & key, unsigned int idx);

    	size_t GetValueCount(const string & section, const string & key);
    	size_t GetValueCount(const string & section, const string & subsection, const string & key);

	const KeyMap & GetKeymap(const string & section, const string & subsection);

    	void Insert(const string & section, const string & subsection, const string & key, const string & value);
    	void Dump();

  	private:
    	ConfObject(ConfObject &);

    protected:
    	Sections	m_sections;
};

#endif
