#include "generator.h"
#include "pcdata.h"
#include "tag.h"
#include "datastructure.h"
#include "id.h"
#include "length.h"
#include "boolean.h"
#include "href.h"
#include "vdata.h"
#include "number.h"
#include "xml_lang.h"
#include "class.h"
#include "contenttype.h"
#include "ialign.h"
#include "talign.h"
#include "xml_space.h"
#include "wrapmode.h"
#include "method.h"
#include "nmtoken.h"
#include "cdata.h"
#include "inifileparser.h"
#include "debug.h"

#include "Ttos.h"

#include <list>
#include <string>

Generator* Generator::sm_generator = NULL;
unsigned int Generator::sm_refcount = 0;

int Generator::m_guid = 0;

Generator* Generator::CreateInterface(FileIO* config, dword seed)
{
	if(sm_generator == NULL)
	  sm_generator = new Generator(config, seed);
	sm_refcount++;
	return sm_generator;
}


Generator::Generator(FileIO* configfile, dword seed)
{
	Debug::Message("Seed : %i\n", seed);
	Rnd::SRand(seed);
	config = INIFileParser::Parse(configfile);

	//m_max_gen_depth = atoi(config->Lookup(SECTION_GLOBAL, KEYNAME_MAXGENDEPTH).c_str());
	//if(m_max_gen_depth == 0) m_max_gen_depth = DEFAULT_MAX_GEN_DEPTH;
	m_written_chars = 0;
	timestamp = "not used";
}


Generator::~Generator (  )
{
	sm_refcount--;
	if (sm_refcount == 0)
	{
	  if (sm_generator) {
	    delete sm_generator;
	    sm_generator = NULL;
	  }
	}
}


std::list< std::string > Generator::build_attributes(std::string tagname)
{
	std::list< std::string > attr;
	attr.clear();

	std::string attr_name;
	std::string attr_value;
	Datastructure* attr_data;
	int i;

	for (i = 0; i < (int)config->GetValueCount(SECTION_DTD, tagname, KEYNAME_REQATTR); i+=2 )
	{
		attr_name = config->Lookup(SECTION_DTD, tagname, KEYNAME_REQATTR, i);
		attr_data = eval_attribute( config->Lookup(SECTION_DTD, tagname, KEYNAME_REQATTR, i+1) );
		attr_value = attr_data->ToString();

		if (config->Lookup(SECTION_GLOBAL, "autorefresh") == "1")
		{
			if (attr_name == "ontimer") {
			  attr_value = config->Lookup(SECTION_GLOBAL, "refreshurl" );
			  //hier muss noch die Timestamp als Parameter angehngt werden!
			  attr_value += "/?ts=" + timestamp;
			}
			if (attr_name == "value") attr_value = "10";
		}

		attr.push_back( " " + attr_name + "=\"" + attr_value  + "\"" );
		m_written_chars += ( (int)attr_name.size() + (int)attr_value.size() + 2 );
	}

	for (i = 0; i < (int)config->GetValueCount(SECTION_DTD, tagname, KEYNAME_IMPLATTR); i+=2 )
	{
		/*
		if (rnd() < 0.5)
		{
			attr_name = config->Lookup(SECTION_DTD, tagname, KEYNAME_IMPLATTR, i);
			attr_data = eval_attribute( config->Lookup(SECTION_DTD, tagname, KEYNAME_IMPLATTR, i+1) );
			attr_value = attr_data->ToString();

			attr.push_back( " " + attr_name + "=\"" + attr_value + "\"" );
			m_written_chars += ( attr_name.size() + attr_value.size() + 2 );
		}
		///////////////////////////////
		*/

		if ( (rnd() < 0.5) || (		((config->Lookup(SECTION_DTD, tagname, KEYNAME_IMPLATTR, i) == "ontimer") ||
									(config->Lookup(SECTION_DTD, tagname, KEYNAME_IMPLATTR, i) == "value")     )
									&& (config->Lookup(SECTION_GLOBAL, "autorefresh") == "1") ) )
		{
			attr_name = config->Lookup(SECTION_DTD, tagname, KEYNAME_IMPLATTR, i);
			attr_data = eval_attribute( config->Lookup(SECTION_DTD, tagname, KEYNAME_IMPLATTR, i+1) );
			attr_value = attr_data->ToString();

			if (config->Lookup(SECTION_GLOBAL, "autorefresh") == "1")
			{
			  if (attr_name == "ontimer") {
			    attr_value = config->Lookup(SECTION_GLOBAL, "refreshurl" );
				//hier muss noch die Timestamp als Parameter angehngt werden!
			    attr_value += "/?ts=" + timestamp;
			  }
			  if (attr_name == "value") attr_value = "10";
			}

			attr.push_back( " " + attr_name + "=\"" + attr_value + "\"" );
			m_written_chars += ( (int)attr_name.size() + (int)attr_value.size() + 2 );
		}
	}

	return attr;
}

void Generator::build_children(Element* elem)
{
	//static unsigned int count = 0;
	std::string rule = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_RULE);

	//if(count++ > m_max_gen_depth) return;

	//! Regeln unterscheiden
	if ( rule == ",+" )
	{
		int childcnt = config->GetValueCount(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN);
		int i=0;
		for ( ; i< childcnt-1; ++i)
		{
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, i);
			double prob = atof( config->Lookup(SECTION_DTD, childname, KEYNAME_PROB).c_str() );

			//! Testen nach einer Parent-Regel
			if (config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname) != "")
			{				
				prob = atof( config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname).c_str() );			
			}

			//! Testen nach einer "entfernten Parent"-Regel
			std::map<string, std::vector<string> > rules = config->GetKeymap(SECTION_FURTHER_PARENTS, childname);



			if ( rnd() < prob)
			{
				//! Testen auf Empty-Tag
				if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
					m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

				if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
					m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );

				m_written_chars += ( (int)childname.size() + 4 + elem->m_spaces+2 );
			}
		}

		std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, i);
		m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );
		m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );

		double pr = 0.75;
		double factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		while ( ((rnd() < pr) && (pr > 0.1)) )
		{
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, i);

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );

			m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );
			pr -= 0.2;
			factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		}

	}


	if ( rule == "," )
	{
		int childcnt = config->GetValueCount(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN);
		for (int i=0 ; i< childcnt; ++i)
		{
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, i);
			double prob = atof( config->Lookup(SECTION_DTD, childname, KEYNAME_PROB).c_str() );
			if ( rnd() < prob)
			{
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );
				m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );
			}
		}
	}
	else if ( rule == "*" )
	{
		//std::cout << "Tag: " << elem->m_tagname << " bearbeiten:\n";

		int childcnt = config->GetValueCount(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN);
		double pr = 0.9;

		double factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;

		while ( (rnd() < pr) && (pr > 0.1) && (factor<1) )
		{
			factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;

			int index = (int) ( rnd() * childcnt );
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, index);
			double prob = atof( config->Lookup(SECTION_DTD, childname, KEYNAME_PROB).c_str() );
			//std::cout << "Kind Nummer " << index << " namens " << childname << " mit Wahrscheinlichkeit " << prob << " erstellen.\n";

			//! Testen nach einer direct Parent-Regel
			if (config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname) != "")
			{				
				prob = atof( config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname).c_str() );				
			}


			//! Testen nach einer "entfernten Parent"-Regel
			std::map<string, std::vector<string> > rules = config->GetKeymap(SECTION_FURTHER_PARENTS, childname);

			//std::cout << "vorhandene Parents: ";
			// ber alle parents gehen und vergleichen ob in den Regeln dieser parent enthalten ist:

			std::map< std::string, bool > parents = elem->get_parents( );
			std::map< std::string, bool >::iterator iter = parents.begin();
			while(iter != parents.end())
			{
				//std::cout << iter->first << "\n";


				std::vector<string> prob_value = rules[iter->first];

				if (prob_value.size() > 0) prob = atof(prob_value[0].c_str());				
				++iter;
			}
			//std::cout << "\n";
			//std::cout << rules.size() << "\n";



			if ( rnd() < prob)
			{
				if (childname == "pcdata")
				{
					m_todo.push_back(new PCData( config->Lookup(SECTION_GLOBAL, KEYNAME_ENCODING ), elem, atol( config->Lookup(SECTION_GLOBAL, KEYNAME_STRINGLENGTH, 0 ).c_str() ), atol( config->Lookup(SECTION_GLOBAL, KEYNAME_STRINGLENGTH, 1 ).c_str() )) );
				}
				else
				{
					if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
						m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

					if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
						m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );

					m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );
				}
			}
			pr-=0.1;
		}
	}
	else if ( rule == "|" )
	{
		int childcnt = config->GetValueCount(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN);

		bool erstellt = false;

		while (!erstellt)
		{
			int index = (int) ( rnd() * childcnt );
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, index);
			double prob = atof( config->Lookup(SECTION_DTD, childname, KEYNAME_PROB).c_str() );

			//! Testen nach einer direct-Parent-Regel
			if (config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname) != "")
			{				
				prob = atof( config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname).c_str() );				
			}
			//! Testen nach einer "entfernten Parent"-Regel
			std::map<string, std::vector<string> > rules = config->GetKeymap(SECTION_FURTHER_PARENTS, childname);

			//std::cout << "vorhandene Parents: ";
			// ber alle parents gehen und vergleichen ob in den Regeln dieser parent enthalten ist:

			std::map< std::string, bool > parents = elem->get_parents( );
			std::map< std::string, bool >::iterator iter = parents.begin();
			while(iter != parents.end())
			{
				//std::cout << (*iter).first << " ";
				std::vector<std::string> prob_value = rules[iter->first];

				if (prob_value.size() > 0) prob = atof(prob_value[0].c_str());				

				++iter;
			}
			//std::cout << "\n";

			if ( rnd() < prob)
			{
				if (childname == "pcdata")
				{
					m_todo.push_back(new PCData( config->Lookup(SECTION_GLOBAL, KEYNAME_ENCODING ), elem, atol( config->Lookup(SECTION_GLOBAL, KEYNAME_STRINGLENGTH, 0 ).c_str() ), atol( config->Lookup(SECTION_GLOBAL, KEYNAME_STRINGLENGTH, 1 ).c_str() )) );
				}
				else
				{
					if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
						m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );
					if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
						m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );
					m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );
				}
				erstellt = true;
			}
		}
	}
	else if ( rule == "|+" )
	{
		int childcnt = config->GetValueCount(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN);

		int index = (int) ( rnd() * childcnt );

		std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, index);
		if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );

		double pr = 0.75;
		double factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		while ( (rnd() < pr) && (pr > 0.1) && (factor<1) )
		{
			index = (int) ( rnd() * childcnt );
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, index);

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );

			m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );
			pr -= 0.2;
			factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		}

	}
	else if ( rule == "*_?_|*")
	{
		int childcnt = config->GetValueCount(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN) - 2;

		double pr = 0.35;
		double factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, 0);
		while ( (rnd() < pr) && (pr > 0.1) && (factor<1) )
		{
			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );
			pr-=0.2;
		}

		childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, 1);

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

			if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
				m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );



		int index = (int) ( rnd() * childcnt ) + 2;

		pr = 0.75;
		factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		while ( (rnd() < pr) && (pr > 0.1) && (factor<1) )
		{
			index = (int) (( rnd() * childcnt ) + 2);
			std::string childname = config->Lookup(SECTION_DTD, elem->m_tagname, KEYNAME_CHILDREN, index);
			double prob = atof( config->Lookup(SECTION_DTD, childname, KEYNAME_PROB).c_str() );

			//std::cout << "card regel: " << childname << "\n";

			//! Testen nach einer direct-Parent-Regel
			if (config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname) != "")
			{				
				prob = atof( config->Lookup(SECTION_DIRECT_PARENTS, elem->m_tagname, childname).c_str() );				
			}
			//! Testen nach einer "entfernten Parent"-Regel
			std::map<string, std::vector<string> > rules = config->GetKeymap(SECTION_FURTHER_PARENTS, childname);

			//std::cout << "vorhandene Parents: ";
			// ber alle parents gehen und vergleichen ob in den Regeln dieser parent enthalten ist:

			std::map< std::string, bool > parents = elem->get_parents( );
			std::map< std::string, bool >::iterator iter = parents.begin();
			while(iter != parents.end())
			{
				// std::cout << iter->first << "\n";
				std::vector<std::string> prob_value = rules[iter->first];

				if (prob_value.size() > 0) prob = atof(prob_value[0].c_str());									
				++iter;
			}

			if (prob < rnd())
			{
				if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "false" )
					m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2) );

				if ( config->Lookup(SECTION_DTD, childname, KEYNAME_EMPTY_TAG) == "true" )
					m_todo.push_back(new Tag(childname, elem, elem->get_parents(), elem->m_spaces+2, true ) );
			}

			m_written_chars += ( childname.size() + 4 + elem->m_spaces+2 );
			pr -= 0.08;
			factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		}

	}
	else if ( true )
	{
	}

}



WML_Tree* Generator::generateTree (  )
{
	Element* current;


	//! lese Anfangstag
	std::string tagname = config->Lookup(SECTION_GLOBAL, KEYNAME_STARTTAG);

	Debug::Message("Generiere Baum!...");
	Debug::Message("Starttag ist :%s " ,tagname.c_str());

	std::list< std::string > attr;

	if (tagname == "pcdata" )
	{
		current = new PCData( config->Lookup(SECTION_GLOBAL, KEYNAME_ENCODING), NULL, atol( config->Lookup(SECTION_GLOBAL, KEYNAME_STRINGLENGTH, 0 ).c_str()  ), atol( config->Lookup(SECTION_GLOBAL, KEYNAME_STRINGLENGTH, 1 ).c_str()  ) );
	}
	else
	{
		attr = build_attributes( tagname );
		std::map< std::string, bool > noparents;
		current = new Tag( tagname, NULL, attr, noparents, 0, false );

		/*
		////////////////////////
		std::list< std::string > attribute;
		attribute.push_back("ontimer=\"http://www.google.de\"");
		Element* timer_karte = new Tag( "card", current, attribute, noparents, 2, false );

		attribute.clear();
		attribute.push_back("value=\"10\"");
		Element* timer = new Tag( "timer", current, attribute, noparents, 4, false );
		timer_karte->addChild( timer );
		current->addChild( timer_karte );
		////////////////////////
		*/

		build_children( current );
	}



	//! WML_Tree mit erstem Element erstellen
	WML_Tree* tree = new WML_Tree( current, config->Lookup(SECTION_GLOBAL, KEYNAME_ENCODING) );

	Debug::Message("Gehe in while..");

	double factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
	while ( factor < 1 )
	{
		while (m_todo.size() > 0)
		{
			attr.clear();
			current = m_todo.front();
			m_todo.pop_front();
			if( current->m_tagname != TAG_PCDATA )
			{
				std::list< std::string > attr = build_attributes( current->m_tagname );
				((Tag*)current)->addAttributes( attr );
				build_children( current );
			}


			if (current->m_parent != NULL)
			{
				current->m_parent->addChild( current );
			}
		}

		factor = (double)m_written_chars / (int)atof( config->Lookup(SECTION_GLOBAL, KEYNAME_MINFILELENGTH).c_str()) ;
		if ( factor < 1 )
		{
			attr.clear();
			attr = build_attributes( "card" );
			std::map< std::string, bool > noparents;
			Element* card = new Tag( "card", tree->m_initTag, attr, noparents, 2, false );
			tree->m_initTag->addChild( card );
			build_children( card );
		}
	}

	Debug::Message("Baum erstellt.");
	Debug::Message("Size: %i", m_written_chars);	

	return tree;
}

int Generator::get_written_bytes (  )
{
	return 0;
}


Datastructure* Generator::eval_attribute ( std::string type )
{
  if ( type == "method" )
    {
      //std::cout << (post|get);
      return new Method();
    }

  if ( type == "nmtoken" )
    {
      //std::cout << "nmtoken";
      return new NMToken();
    }
  if ( type == "cdata" )
    {
      //std::cout << "CData";
      return new CData();
    }
  if ( type == "length" )
    {
      //std::cout << "length";
      return new Length();
    }
  if ( type == "boolean" )
    {
      //std::cout << "boolean";
      return new Boolean();
    }
  if ( type == "href" )
    {
      //std::cout << "href";
      return new Href();
    }
  if ( type == "vdata" )
    {
      //std::cout << "vdata";
      return new Vdata();
    }
  if ( type == "number" )
    {
      //std::cout << "number";
      return new Number();
    }
  if ( type == "xml_lang" )
    {
      //std::cout << "xml:lang";
      return new Xml_lang();
    }
  if ( type == "class" )
    {
      //std::cout << "class";
      return new Class();
    }
  if ( type == "id" )
    {
	//std::cout << "id";
      return new Id();
    }
  if ( type == "contenttype" )
    {
      //std::cout << "contenttype";
      return new Contenttype();
    }
  if ( type == "ialign" )
    {
      //std::cout << "ialign";
      return new IAlign();
    }
  if ( type == "talign" )
    {
      //std::cout << "talign";
      return new TAlign();
    }
  if ( type == "wrapmode" )
    {
      //std::cout << "wrapmode";
      return new Wrapmode();
    }
  if ( type == "xml_space" )
    {
      //std::cout << "xml:space";
      return new Xml_space();
    }

  Debug::Error("Kann Datentyp %s nicht interpretieren !! Terminating\n", type.c_str() );
  exit(1);
}


double Generator::rnd()
{
	return (double)Rnd::Rand() / (double)RND_MAX;
}

int Generator::get_GUID()
{
	return ++m_guid;
}

void Generator::setTimestamp (std::string ts ) {
  //std::cout << ts;
  timestamp = ts;
}
