#ifndef GENERATOR_H
#define GENERATOR_H

#include "plattform.h"
#include "confobject.h"
#include "defines.h"
#include "rnd.h"
#include "fileio.h"
#include "wml_tree.h"
#include "element.h"
#include "datastructure.h"
#include <list>

//! Die Definitionen fr Sektionsnamen
#define	SECTION_GLOBAL			"global"
#define SECTION_DTD				"dtd"
#define SECTION_DIRECT_PARENTS	"direct_parents"
#define SECTION_FURTHER_PARENTS	"further_parents"

//! Die Definitionen fr Schlsselnamen
#define KEYNAME_MAXGENDEPTH		"maxchildrengenerationdepth"
#define KEYNAME_STARTTAG		"starttag"
#define KEYNAME_REQATTR			"requiredattributes"
#define KEYNAME_IMPLATTR		"impliedattributes"
#define KEYNAME_CHILDREN		"children"
#define KEYNAME_EMPTY_TAG		"empty"
#define KEYNAME_PROB			"prob"
#define KEYNAME_RULE			"rule"
#define KEYNAME_ENCODING		"encoding"
#define KEYNAME_STRINGLENGTH	"averagelengthstring"
#define KEYNAME_MINFILELENGTH	"minlengthfile"

//! Die Definitionen fr Sonderwerte
#define	DEFAULT_MAX_GEN_DEPTH	10000
#define TAG_PCDATA				"pcdata"


//! Die Klasse Generator reprsentiert den Generator der Elemente des WML\_Trees. Er erzeugt alle Elemente und Attribute, die spter in dem Dokument erscheinen. Dazu bercksichtigt er Regeln und Konfigurationen nach den Angaben des Benutzers in einer eventuell vorhandenen Konfigurationsdatei.
class Generator
{
 public:
  //! Die Liste mit den noch abzuhandelnden Elemente bevor diese in den Baum eingefgt werden knnen
  std::list< Element* > m_todo;

  //! Funktion zum Erstellen einer Instanz des Generators (Singelton-Eigenschaft). Wenn bereits eine Instanz besteht wird eine Referenz darauf zurckgeliefert.
  static Generator* CreateInterface(FileIO* config = NULL, dword seed = 0);
  //! Der Destruktor des Generators
  ~Generator();

  //! Funktion zum Evaluieren von Attributswerten. Bei gegebenen Textstring wird eine dazu gehrige Datenstruktur erstellt und geliefert.
  Datastructure* eval_attribute ( std::string type );

  //! In dieser Funktion wird unter Bercksichtigung der vom Benutzer definierten Regeln der Dokumenten-Baum erstellt. Dies ist das Kernstck des Generators.
  WML_Tree* generateTree();

  //! Liefert die Anzahl der schon geschriebenen Bytes zurck, so dass die Dateigrssenangaben des Benutzers bercksichtigt werden knnen.
  int get_written_bytes();

  //! Hier findet die Berechnung einer fr das Dokument eindeutigen Identifikationsnummer (GUID) statt.
  static int get_GUID();

  //! Berechnet mit Hilfe der Klasse Rnd eine Zufallszahl zwischen 0 und 1.
  static double rnd();

  //! setzt die timestamp
  void setTimestamp ( std::string ts );

 private:
  //! Speicher der zuletzt erzeugten GUID
  static int m_guid;
  //! Speicher der Ausgabelnge
  int m_written_chars;

  //unsigned int m_max_gen_depth;
  //! Referenz auf die Konfiguration zum Nachschlagen der Benutzerwerte
  ConfObject* config;
  //! Zhler der Referenzen auf dieses Objekt
  static unsigned int sm_refcount;
  //! Referenz auf die eine Instanz des Objektes
  static Generator*	sm_generator;

  //! Funktion zur Erstelltung der Attribute des angegebenen Elementes
  std::list< std::string >  build_attributes(std::string tagname);

  //! Funktion zur Erstelltung der Kinder des angegebenen Elementes
  void build_children(Element* elem);

  //! Konstruktor des Generators mit Defaultwerten
  Generator(FileIO* config = NULL, dword seed = 0);

  //! speichert die timestamp
  std::string timestamp;
};

#endif

