#include "sqltools.h"

wmlgenDB::wmlgenDB (std::string host, std::string user, std::string passwd, std::string db) {
  connected = false;
  try {
    mydb = new MySQLaux::Database(host, user, passwd, db);
  }
  catch (MySQLaux::Error &myerror) {
    std::cerr << "Fehler beim Verbinden zur DB (" << myerror.asString () << "\n";
    return;
  }
  connected = true;
}

bool wmlgenDB::isConnected (){
  return connected;
}

MySQLaux::Table*  wmlgenDB::doQuery (const std::string &query) {
  MySQLaux::Table* mytable;
  if (!connected)
    return NULL;
  try {
    mytable = new MySQLaux::Table(mydb->query (query));
  }
  catch (MySQLaux::Error &myerror) {
    std::cerr << "Fehler bei Query (" <<  myerror.asString () << ")\n";
    return NULL;
  }
  catch (MySQLaux::MissingFeature &myerror) {
    std::cerr << "Fehler bei Query (" << myerror.asString () << "\n";
    return NULL;
  }
  return mytable;
}


bool wmlgenDB::printQuery (const std::string &query) {
  if (!connected)
    return false;
  try {
    MySQLaux::Table mytable = mydb->query (query);

    for (uint i = 0; i < mytable.size () ; i++) {
      for (uint j = 0 ; j <  mytable[i].size () ; j++) {
	std::string tmp;
	//this->escapeString (mytable[i][j].to_string (), tmp);
	tmp = mytable[i][j].to_string ();
	std::cout << tmp << " | ";
        std::cout << endl;
      }
    }
  }
  catch (MySQLaux::Error &myerror) {
    cout << "Fehler bei Query (" << myerror.asString () << "\n";
    return false;
  }
  catch (MySQLaux::MissingFeature &myerror) {
    cout << "Fehler bei Query (" << myerror.asString () << "\n";
    return false;
  }
  return true;
}

void wmlgenDB::escapeString (const std::string &source, std::string &target) {
  //unsigned long   STDCALL mysql_escape_string(char *to,const char *from,
  //                                        unsigned long from_length);
  char *buffer;
  buffer = (char*) malloc (sizeof (char) * (source.length() + 1)*2);

  mysql_escape_string (buffer, source.c_str(), source.length() );
  target = std::string (buffer);
  free (buffer);
  assert (target.find ("'",0) == string::npos);
}


/* *************************************************************************** */

