#include <cppunit/TestCase.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TestCaller.h>
#include <unistd.h>
#include <iostream>
#include <map>
#include <list>

#include "wml_interface.h"
#include "tag.h"
#include "length.h"
#include "vdata.h"
#include "href.h"
#include "number.h"
#include "xml_lang.h"
#include "xml_space.h"
#include "class.h"
#include "id.h"
#include "ialign.h"
#include "talign.h"
#include "wrapmode.h"
#include "contenttype.h"
#include "wml_tree.h"

class TagTest : public CppUnit::TestCase {
 public: 
  TagTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    std::map<std::string, bool> parents;
    std::list<std::string> attr;
    attr.push_back("name=wert");
    Tag* tag = new Tag("p", NULL, attr, parents, 0, true);
    CPPUNIT_ASSERT ( tag->ToString() == "<p name=wert/>\n" );
    attr.pop_back();
    Tag* tag2 = new Tag("p2", NULL, attr, parents, 0, false);
    tag2->addChild(tag);
    CPPUNIT_ASSERT ( tag2->ToString() == "<p2>\n<p name=wert/>\n</p2>\n" );
    delete tag;
    delete tag2;
  }

  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "TagTest" );
    suiteOfTests->addTest( new TagTest("TagTest") );
    return suiteOfTests;
  }
};

class LengthTest : public CppUnit::TestCase {
 public: 
  LengthTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Length();
    CPPUNIT_ASSERT ( data->ToString() != "" );
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "LengthTest" );
    suiteOfTests->addTest( new LengthTest("LengthTest") );
    return suiteOfTests;
  }
};


class VdataTest : public CppUnit::TestCase {
 public: 
  VdataTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Vdata();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "VdataTest" );
    suiteOfTests->addTest( new VdataTest("VdataTest") );
    return suiteOfTests;
  }
};


class HrefTest : public CppUnit::TestCase {
 public: 
  HrefTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Href();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "HrefTest" );
    suiteOfTests->addTest( new HrefTest("HrefTest") );
    return suiteOfTests;
  }
};

class Xml_langTest : public CppUnit::TestCase {
 public: 
  Xml_langTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Xml_lang();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "Xml_langTest" );
    suiteOfTests->addTest( new Xml_langTest("Xml_langTest") );
    return suiteOfTests;
  }
};

class Xml_spaceTest : public CppUnit::TestCase {
 public: 
  Xml_spaceTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Xml_space();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "Xml_spaceTest" );
    suiteOfTests->addTest( new Xml_spaceTest("Xml_spaceTest") );
    return suiteOfTests;
  }
};

class ClassTest : public CppUnit::TestCase {
 public: 
  ClassTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Class();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "ClassTest" );
    suiteOfTests->addTest( new ClassTest("ClassTest") );
    return suiteOfTests;
  }
};

class IdTest : public CppUnit::TestCase {
 public: 
  IdTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Id();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "IdTest" );
    suiteOfTests->addTest( new IdTest("IdTest") );
    return suiteOfTests;
  }
};

class IAlignTest : public CppUnit::TestCase {
 public: 
  IAlignTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new IAlign();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "IAlignTest" );
    suiteOfTests->addTest( new IAlignTest("IAlignTest") );
    return suiteOfTests;
  }
};

class TAlignTest : public CppUnit::TestCase {
 public: 
  TAlignTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new TAlign();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "TAlignTest" );
    suiteOfTests->addTest( new TAlignTest("TAlignTest") );
    return suiteOfTests;
  }
};

class WrapmodeTest : public CppUnit::TestCase {
 public: 
  WrapmodeTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Wrapmode();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "WrapmodeTest" );
    suiteOfTests->addTest( new WrapmodeTest("WrapmodeTest") );
    return suiteOfTests;
  }
};

class ContentTypeTest : public CppUnit::TestCase {
 public: 
  ContentTypeTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    Datastructure* data = new Contenttype();
    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) 
      delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "ContentTypeTest" );
    suiteOfTests->addTest( new ContentTypeTest("ContentTypeTest") );
    return suiteOfTests;
  }
};

class WmlTreeTest : public CppUnit::TestCase {
 public: 
  WmlTreeTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    std::map<std::string, bool> parents;
    std::list<std::string> attr;
    attr.push_back("name=wert");
    Tag* tag = new Tag("p", NULL, attr, parents, 0, true);
    WML_Tree* tree = new WML_Tree(tag, "iso-8859-1");
    CPPUNIT_ASSERT (tree->ToString() == "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.3//EN\"\n\"http://www.wapforum.org/DTD/wml13.dtd\">\n<p name=wert/>\n");
    attr.pop_back();
    Tag* tag2 = new Tag("p2", NULL, attr, parents, 0, false);
    tag2->addChild(tag);
    if (tree) delete tree;
    tree = new WML_Tree(tag2, "iso-8859-1");
    CPPUNIT_ASSERT (tree->ToString() == "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.3//EN\"\n\"http://www.wapforum.org/DTD/wml13.dtd\">\n<p2>\n<p name=wert/>\n</p2>\n");
    if (tag) delete tag;
    if (tag2) delete tag2;
    if (tree) delete tree;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "WmlTreeTest" );
    suiteOfTests->addTest( new WmlTreeTest("WmlTreeTest") );
    return suiteOfTests;
  }
};

class ConfObjectTest : public CppUnit::TestCase {
 public: 
  ConfObjectTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    ConfObject* conf = new ConfObject();
    conf->Insert("section", "subsection", "name", "value");
    CPPUNIT_ASSERT(conf->Exist("section", "subsection", "name") == true);
    CPPUNIT_ASSERT(conf->Exist("section", "subsectio", "name") == false);
    CPPUNIT_ASSERT(conf->Lookup("section", "subsection", "name") == "value");
    CPPUNIT_ASSERT(conf->Lookup("section", "name") == "");
    if (conf) delete conf;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "ConfObjectTest" );
    suiteOfTests->addTest( new ConfObjectTest("ConfObjectTest") );
    return suiteOfTests;
  }
};

class Wml_InterfaceTest : public CppUnit::TestCase {
 public: 
  Wml_InterfaceTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    WML_Interface* wml = new WML_Interface("default.cfg");
    CPPUNIT_ASSERT(wml != NULL);
    if (wml) delete wml;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "Wml_InterfaceTest" );
    suiteOfTests->addTest( new Wml_InterfaceTest("Wml_InterfaceTest") );
    return suiteOfTests;
  }
};

class GeneratorTest : public CppUnit::TestCase {
 public: 
  GeneratorTest( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {
    WML_Interface* wml = new WML_Interface("default.cfg");
    CPPUNIT_ASSERT (wml != NULL);
    wml->build_tree();
    //eventuell hier mit wine den validator aufrufen?
    CPPUNIT_ASSERT (wml->output() != "");
    if (wml) delete wml;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "GeneratorTest" );
    suiteOfTests->addTest( new GeneratorTest("GeneratorTest") );
    return suiteOfTests;
  }
};


/*
class Test : public CppUnit::TestCase {
 public: 
  Test( std::string name ) : CppUnit::TestCase( name ) {}

  void runTest() {

    CPPUNIT_ASSERT ( data->ToString() != "");
    if (data) delete data;
  }
  static CppUnit::Test *suite() {
    CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( "Test" );
    suiteOfTests->addTest( new Test("Test") );
    return suiteOfTests;
  }
};
*/

